package com;

import com.fasterxml.jackson.annotation.JsonFilter;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.JsonStreamContext;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.PropertyWriter;
import com.fasterxml.jackson.databind.ser.impl.SimpleBeanPropertyFilter;
import com.fasterxml.jackson.databind.ser.impl.SimpleFilterProvider;

import java.util.*;

import static java.lang.System.out;

public class Test {
    public static void main(String[] args) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        SimpleFilterProvider provider = new SimpleFilterProvider().addFilter("myFilter", new MyFilter());
        ObjectWriter writer = mapper.writer(provider);

        out.println();
        out.println("== Item with ArrayList ==");
        out.println(writer.writeValueAsString(new Item(new ArrayList<String>(), "ID-1")));

        out.println("\n");
        out.println("== Item with HashSet == ");
        out.println(writer.writeValueAsString(new Item(new HashSet<String>(), "ID-2")));
    }


    public static class MyFilter extends SimpleBeanPropertyFilter {
        @Override
        public void serializeAsField(Object pojo, JsonGenerator jgen, SerializerProvider provider, PropertyWriter writer) throws Exception {
            super.serializeAsField(pojo, jgen, provider, writer);

            JsonStreamContext ctx = jgen.getOutputContext();
            out.printf("Current Name: %s, Current Value: %s\n", ctx.getCurrentName(), ctx.getCurrentValue().getClass().getName());
        }
    }


    @JsonFilter("myFilter")
    public static class Item {
        private Collection<String> set;
        private String id;

        public Item(Collection<String> set, String id) {
            this.set = set;
            this.id = id;
        }

        public Collection<String> getSet() {
            return set;
        }

        public String getId() {
            return id;
        }
    }
}
